#!/usr/bin/perl
# sutcheck.pl
# 98/06/04 gar
# 
# Zum Ueberpruefen und festschreiben der Protokolle der
# Singleusertests.

use Env;
use Cwd;
use Getopt::Std;
use File::Copy;
require "$TOOL/bin/sutvars$TOOLEXT";

if (!getopts('hpr') || $opt_h) {
    print <DATA>;
    exit 1;
}
if ($#ARGV == -1){
   print <DATA>;
   exit 1;
}

$VERSION = shift;
if ( $VERSION !~ /^(fast|quick|slow)/i) {
    print <DATA>;
    exit 1;
}

$PortModus = $opt_p;
$RefModus = $opt_r;
chdir($SUT) || die "Can't change directory: $!\n";
$ProtDir = "$SUT/$VERSION";

if  ( $#ARGV == -1 ) {

  opendir(DIR_IN, $ProtDir);
  @Files = grep /^[A-Z].*\.pdiff/i, readdir(DIR_IN);
  closedir DIR_IN;

} else {

  @Files = @ARGV;
}


# $EDITOR="less";

unless ( $opt_p ) {

  system("cls");
  print "... folgende Differenzen werden bearbeitet:\n\n";

  $i = 1;
  foreach (@Files) {
    print "$_\n";
    $i++;
    if ( $i == 20) {
	print "\nmore ...\n";
	$ANSW = <STDIN>;
        $i = 1;
    }
  }
  <STDIN> if ( $i < 20 );
  print "\n\n";
}

map { s/\.pdiff//i; tr/[a-z]/[A-Z]/} @Files;

foreach $FN (@Files) {
  $Title="$FN";
  system("cls");
  system ("set_title 'sutcheck: $Title'");
  if ( -f "$ProtDir/$FN.pdiff" ) {
     # &gar -> Vergleich mit vorher erstellten ref-Protokoll 
     if ($VERSION =~ /^(fast|quick)/i){
        if (-f "$SUT/$FN.tpunx") {
           @statinfo_org=stat "$SUT/$FN.tpunx";
        }
        elsif (-f "$SUT/$FN.punix"){
           @statinfo_org=stat "$SUT/$FN.punix";
        }
        else{
           print "\n$SUT/$FN.punix or $FN.tpunx not found";
           $ANSW = <STDIN>;
           next;
        } 
     }
     # slow version
     else{
        if (-f "$SUT/$FN.punix") {
           @statinfo_org=stat "$SUT/$FN.punix";
        }
        else{
           print "\n$SUT/$FN.punix or $FN.tpunx not found";
           $ANSW = <STDIN>;
           next;
        }
     }
     @ARGV = ("-e","$EDITOR", "-r", "-s"); 
     if ( $RefModus ) {
        if (-f "$ProtDir/$FN.pmach"){
           @statinfo_mach=stat "$ProtDir/$FN.pmach";
           # comp modification time
           if ($statinfo_org[9]>$statinfo_mach[9]){
              move("$ProtDir/$FN.pmach", "$ProtDir/$FN.old_pmach");
              system ("set_title 'sutcheck: $Title - reference protocol older then command file (ignored)'");
              push @ARGV, "$ProtDir/$FN.pdiff"; 
           }
           else{
              system("$DIFF $ProtDir/$FN.pmach $ProtDir/$FN.pdiff > $ProtDir/$FN.mdiff");
              if (-s "$ProtDir/$FN.mdiff"){
                 push @ARGV, "$ProtDir/$FN.mdiff", "$ProtDir/$FN.pdiff";
                 system ("set_title 'sutcheck: $Title - protocol and the reference protocol differed ($FN.mdiff)'");
              }  
              else{
                 unlink "$ProtDir/$FN.mdiff";
                 unlink "$ProtDir/$FN.pdiff";
                 unlink "$ProtDir/$FN.prot";
                 print "\nno differences between $FN.pdiff $FN.pmach";
                 $ANSW = <STDIN>; 
                 next;   
              }
           }
        }
        else{
	   push @ARGV, "$ProtDir/$FN.pdiff"; 
        }
     }
     else{
        push @ARGV, "$ProtDir/$FN.pdiff";
     }
     
     if ( $PortModus ) {
         push @ARGV, "$ProtDir/$FN.prot";
     }
     
     do "$TOOL/bin/opendoc$TOOLEXT";
     if ( $@ ) { die "Error while executing opendoc$TOOLEXT: $@" }


     if ( -f "$FN.punix" ) {
	@statinfo = stat "$FN.punix";
	( $r_size, $r_mtime ) = (\7, \9);
	print "@statinfo[$$r_size] ",
              scalar localtime(@statinfo[$$r_mtime]),
              " $FN.punix\n";
     }

     if ( -f "$FN.tpunx" ) {
	@statinfo = stat "$FN.tpunx";
	( $r_size, $r_mtime ) = (\7, \9);
	print "@statinfo[$$r_size] ",
              scalar localtime(@statinfo[$$r_mtime]),
              " $FN.tpunx\n";
     }

     if ( -f "$ProtDir/$FN.mach" ) {
	@statinfo = stat "$ProtDir/$FN.mach";
	( $r_size, $r_mtime ) = (\7, \9);
	print "@statinfo[$$r_size] ",
              scalar localtime(@statinfo[$$r_mtime]),
              " $FN.mach\n";
     }

     if ( -f "$ProtDir/$FN.prot" ) {
	@statinfo = stat "$ProtDir/$FN.prot";
	( $r_size, $r_mtime ) = (\7, \9);
	print "@statinfo[$$r_size] ",
              scalar localtime(@statinfo[$$r_mtime]),
              " $FN.prot\n";
     }

     if ( -f "$ProtDir/$FN.pdiff" ) {
	@statinfo = stat "$ProtDir/$FN.pdiff";
	( $r_size, $r_mtime ) = (\7, \9);
	print "@statinfo[$$r_size] ",
              scalar localtime(@statinfo[$$r_mtime]),
              " $FN.pdiff\n";
     }


   # open(FILE_IN, "<$AllSut");
   # while(<FILE_IN>) { do { print $_; last; } if /$FN/}
   # close(FILE_IN);
    
    $ANSW="x";
    if ( $RefModus ){
       until ( ($ANSW =~ /^(q|r|n)/i) || $ANSW eq "" ) {
          print "\n==> Quit / Remove / New Reference / Nothing";
          print "\n==> Command : (Q/R/N/<CR>)";
          $ANSW = <STDIN>;
          chop($ANSW);
       }
    }
    else{
       until (($ANSW =~ /^(q|r)/i) || $ANSW eq "" ) {
          print "\n==> Quit / Remove / Nothing";
          print "\n==> Command : (Q/R/<CR>)";
          $ANSW = <STDIN>;
          chop($ANSW);
       }
    }

    CASE: {

	if ( $ANSW =~ /r/i ) {

          unlink ("$ProtDir/$FN.pdiff", "$ProtDir/$FN.prot");
          print "$FN.pdiff und $FN.prot removed\n";
          last CASE;
	}

        if ( $ANSW =~ /q/i ) {

          print "QUIT check-sut.\n";
          system ("set_title ' '");
          exit;
          last CASE;
        }
	# &gar ->
        if ( $ANSW =~ /n/i && $RefModus) {
           if ( -f "$ProtDir/$FN.pmach"){
              move ("$ProtDir/$FN.pmach", "$ProtDir/$FN.old_pmach");
              print "old ref-file copied to $FN.old_pmach\n"
           }    
           move ("$ProtDir/$FN.pdiff", "$ProtDir/$FN.pmach");
           unlink ("$ProtDir/$FN.prot");
           print "$ProtDir/$FN.pdiff is the new ref-file\n";
        }
        # <- &gar	
        if ( $ANSW eq "" ) {
          print "$FN.prot und $FN.pdiff not removed\n";
          last CASE;
	}
    }

    do "$TOOL/bin/closedoc$TOOLEXT";
    if ( $@ ) { die "Error while executing closedoc$TOOLEXT: $@" }
  }
}
print "\n";
system ("set_title ' '");


__DATA__


  USAGE:  sutcheck  [-prh] <version> [<protname> ...]

        tool to check the SUT result

  OPTIONS:
     <version>:  fast|quick|slow
        -h    :  this help screen
        -p    :  port modus (only pdiff files)
        -r    :  reference modus (comparison with other ref-files)
